import cloneDeep from "clone-deep";
import cssToRN from "css-to-react-native";
import * as React from "react";
const __reactCreateElement__ = React.createElement;
import * as RN from "react-native";
const emptyObj = {};
export function createStyles(options = emptyObj) {
  var _options$themes, _options$tokens;

  const themes = (_options$themes = options.themes) !== null && _options$themes !== void 0 ? _options$themes : emptyObj;
  let currentTheme = "default";
  const tokens = {
    default: (_options$tokens = options.tokens) !== null && _options$tokens !== void 0 ? _options$tokens : emptyObj
  };

  if (options.themes) {
    const themeNames = Object.keys(options.themes);
    currentTheme = themeNames[0];

    for (const themeName of themeNames) {
      tokens[themeName] = mergeTokens(cloneDeep(tokens.default), themes[themeName]);
    }

    const colorScheme = RN.Appearance.getColorScheme();

    if (themeNames.length === 2 && "light" in tokens && "dark" in tokens && colorScheme && colorScheme in themes) {
      currentTheme = colorScheme;
    }
  }

  const isAutoThemeable = Object.values(tokens).length === 3 && "light" in tokens && "dark" in tokens;

  function cls(literals, ...placeholders) {
    const styles = compileStyles(compileLiterals(literals, ...placeholders), tokens[currentTheme]);

    if (process.env.NODE_ENV !== "production") {
      return Object.freeze(styles);
    }

    return styles;
  }

  const styles = Object.assign(function styles(styleMap) {
    // style('text', {})
    function style(...args) {
      const numArgs = args.length;
      let sheet = styleMap.default ? compileStyles(styleMap.default, tokens[currentTheme]) : {};

      if (numArgs === 1 && typeof args[0] === "string") {
        sheet = { ...sheet,
          ...compileStyles(styleMap[args[0]], tokens[currentTheme])
        };
      } else if (numArgs > 0) {
        let i = 0;
        let arg;
        sheet = { ...sheet
        };

        for (; i < numArgs; i++) {
          arg = args[i];

          if (typeof arg === "string") {
            Object.assign(sheet, compileStyles(styleMap[arg], tokens[currentTheme]));
          } else if (typeof arg === "object") {
            for (const key in arg) if (arg[key]) Object.assign(sheet, compileStyles(styleMap[key], tokens[currentTheme]));
          }
        }
      }

      return process.env.NODE_ENV !== "production" ? Object.freeze(sheet) : sheet;
    }

    style.styles = styleMap;
    return style;
  }, {
    one(literals, ...placeholders) {
      let styles = compileStyles(compileLiterals(literals, ...placeholders), tokens[currentTheme]);

      if (process.env.NODE_ENV !== "production") {
        styles = Object.freeze(styles);
      }

      return Object.assign(function oneStyle(createStyle) {
        if (!createStyle && createStyle !== void 0) return emptyObj;
        return styles;
      }, {
        styles
      });
    },

    cls,

    lazy(lazyFn) {
      return function (value) {
        if (value === undefined) return emptyObj;
        let styles = compileStyles(lazyFn(value), tokens[currentTheme]);

        if (process.env.NODE_ENV !== "production") {
          styles = Object.freeze(styles);
        }

        return styles;
      };
    },

    join(...css) {
      return cls("".concat(...css));
    },

    tokens,
    themes
  });
  const DashContext = /*#__PURE__*/React.createContext({
    styles,
    theme: currentTheme,

    setTheme(theme) {
      /* istanbul ignore next */
      console.error("DashContext was consumed outside of a Provider");
    }

  });
  /**
   * A hook for consuming dash context from the provider
   */

  function useDash() {
    return React.useContext(DashContext);
  }

  function styled(Component, styles) {
    function compileRecursiveStyles(style, tokens) {
      function _ref(s) {
        return compileRecursiveStyles(s, tokens);
      }

      if (typeof style === "function" || typeof style === "string") {
        return compileStyles(style, tokens);
      } else if (Array.isArray(style)) {
        return Object.assign({}, ...style.map(_ref));
      }

      return style || {};
    }

    const RefForwardingComponent = /*#__PURE__*/React.forwardRef(function StyledComponent({
      style,
      ...props
    }, ref) {
      const {
        theme
      } = useDash();
      const baseStyle = styles ? compileStyles(styles, tokens[theme]) : void 0;
      const outerStyle = typeof style === "function" || typeof style === "string" ? compileStyles(style, tokens[theme]) : Array.isArray(style) ? compileRecursiveStyles(style, tokens[theme]) : style;
      return /*#__PURE__*/__reactCreateElement__(Component, { ...props,
        style: outerStyle && baseStyle ? { ...baseStyle,
          ...outerStyle
        } : outerStyle ? outerStyle : baseStyle,
        ref: ref
      });
    }); // if (process.env.NODE_ENV !== "production") {
    //   RefForwardingComponent.displayName = `styled(${
    //     Component.displayName ?? (Component.name || "Component")
    //   })`;
    // }

    return RefForwardingComponent;
  }

  function wrapStyled(Component) {
    return function styledWrapper(literals, ...placeholders) {
      return styled(Component, Array.isArray(literals) ? compileLiterals(literals, ...placeholders) : literals);
    };
  }

  styled.ActivityIndicator = wrapStyled(RN.ActivityIndicator);
  styled.DrawerLayoutAndroid = wrapStyled(RN.DrawerLayoutAndroid);
  styled.FlatList = wrapStyled(RN.FlatList);
  styled.Image = wrapStyled(RN.Image);
  styled.ImageBackground = wrapStyled(RN.ImageBackground);
  styled.KeyboardAvoidingView = wrapStyled(RN.KeyboardAvoidingView);
  styled.Modal = wrapStyled(RN.Modal);
  styled.NavigatorIOS = wrapStyled(RN.NavigatorIOS);
  styled.RecyclerViewBackedScrollView = wrapStyled(RN.RecyclerViewBackedScrollView);
  styled.RefreshControl = wrapStyled(RN.RefreshControl);
  styled.SafeAreaView = wrapStyled(RN.SafeAreaView);
  styled.ScrollView = wrapStyled(RN.ScrollView);
  styled.SectionList = wrapStyled(RN.SectionList);
  styled.SnapshotViewIOS = wrapStyled(RN.SnapshotViewIOS);
  styled.Switch = wrapStyled(RN.Switch);
  styled.Text = wrapStyled(RN.Text);
  styled.TextInput = wrapStyled(RN.TextInput);
  styled.TouchableHighlight = wrapStyled(RN.TouchableHighlight);
  styled.TouchableNativeFeedback = wrapStyled(RN.TouchableNativeFeedback);
  styled.TouchableOpacity = wrapStyled(RN.TouchableOpacity);
  styled.TouchableWithoutFeedback = wrapStyled(RN.TouchableWithoutFeedback);
  styled.View = wrapStyled(RN.View);
  return {
    styles,
    styled,
    useDash,

    DashProvider({
      theme: controlledTheme,
      defaultTheme,
      onThemeChange,
      children
    }) {
      const colorScheme = RN.useColorScheme();
      const didMount = React.useRef(false);
      const [userTheme, setTheme] = React.useState(() => {
        if (controlledTheme) {
          currentTheme = controlledTheme;
          return controlledTheme;
        }

        if (defaultTheme) {
          currentTheme = defaultTheme;
          return defaultTheme;
        }

        if (colorScheme && colorScheme in tokens && isAutoThemeable) {
          currentTheme = colorScheme;
          return colorScheme;
        }

        return currentTheme;
      });
      const theme = controlledTheme !== null && controlledTheme !== void 0 ? controlledTheme : userTheme;
      currentTheme = theme;
      const storedOnChange = React.useRef(onThemeChange);
      React.useLayoutEffect(() => {
        storedOnChange.current = onThemeChange;
      });
      React.useLayoutEffect(() => {
        if (controlledTheme && controlledTheme !== userTheme) {
          setTheme(controlledTheme);
        }
      }, [controlledTheme, userTheme]);
      React.useLayoutEffect(() => {
        if (didMount.current && !controlledTheme && colorScheme && colorScheme !== theme && isAutoThemeable) {
          var _storedOnChange$curre;

          currentTheme = colorScheme;
          setTheme(currentTheme);
          (_storedOnChange$curre = storedOnChange.current) === null || _storedOnChange$curre === void 0 ? void 0 : _storedOnChange$curre.call(storedOnChange, currentTheme);
        }

        didMount.current = true;
      }, [colorScheme, controlledTheme]);
      return /*#__PURE__*/__reactCreateElement__(DashContext.Provider, {
        value: React.useMemo(() => ({
          styles,
          theme,

          setTheme(nextTheme) {
            var _storedOnChange$curre2;

            if (nextTheme !== theme) (_storedOnChange$curre2 = storedOnChange.current) === null || _storedOnChange$curre2 === void 0 ? void 0 : _storedOnChange$curre2.call(storedOnChange, nextTheme);
            setTheme(nextTheme);
          }

        }), [theme]),
        children: children
      });
    }

  };
}
/**
 * A utility function that will compile style objects and callbacks into CSS strings.
 *
 * @param styles - A style callback, object, or string
 * @param tokens - A map of CSS tokens for style callbacks
 */

export function compileStyles(styles, tokens) {
  const value = typeof styles === "function" ? styles(tokens) : styles;

  if (typeof value !== "string") {
    return value || {};
  }

  const cachedStyle = compileCache.get(value);
  if (cachedStyle) return cachedStyle;
  const stylePairs = [];
  const splitValue = value.split(";");

  for (let i = 0; i < splitValue.length; i++) {
    // Get prop name and prop value
    const match = propertyValuePattern.exec(splitValue[i]);
    if (match !== null) stylePairs.push([match[1], match[2]]);
  }

  try {
    const style = cssToRN(stylePairs);
    compileCache.set(value, style);
    return style;
  } catch (error) {
    const msg = error.message;

    if (msg.includes("Failed to parse declaration")) {
      const values = msg.replace("Failed to parse declaration ", "").replace(/"/g, "").trim().split(":");
      console.error(`'${values[0]}' shorthand property requires units for example - ${values[0]}: 20px or ${values[0]}: 10px 20px 40px 50px`);
    }

    const style = {};
    compileCache.set(value, style);
    return style;
  }
}
const compileCache = /*#__PURE__*/new Map();

function mergeTokens(target, source) {
  for (const key in source) {
    var _target$key;

    const value = source[key];
    target[key] = typeof value === "object" && value !== null && !Array.isArray(value) ? mergeTokens((_target$key = target[key]) !== null && _target$key !== void 0 ? _target$key : {}, value) : value;
  }

  return target;
}

const propertyValuePattern = /\s*([^\s]+)\s*:\s*(.+?)\s*$/;

function compileLiterals(literals, ...replacements) {
  return Array.isArray(literals) ? literals.reduce((curr, next, i) => curr + next + (replacements[i] || ""), "") : literals;
}